/*
 Pilot Display
 
 Copyright (c) 2014 Dmitry Pakhomenko.
 dmitryp@magictale.com
 http://magictale.com
 
 This code is in the public domain.
 */

#ifndef PilotDisplay_h
#define PilotDisplay_h

const int slaveSelectPin = 4;

#define MAX6957_GLOBAL_CURRENT_REG 0x2
#define MAX6957_CONFIG_REG 0x4
#define MAX6957_DISP_TEST_REG 0x7

#define MAX_LED_BRIGHTNESS 13

#define MAX_E 9
#define E_IDLE 0xF
#define MAX_EGT 4
#define MAX_EDR 10
#define DELAY_IN_TEST_MS 20

#define PEGASUS_OLDER_ENGINE_ID 0x3
#define OLYMPUS_OLDER_ENGINE_ID 0x6
#define MERCURY_OLDER_ENGINE_ID 0x7
#define MERCURY_ENGINE_ID 0x8
#define PEGASUS_ENGINE_ID 0x10
#define OLYMPUS_ENGINE_ID 0x18
#define TITAN_ENGINE_ID 0x20
#define NIKE_ENGINE_ID 0x28
#define LYNX_ENGINE_ID 0x30

#define MERCURY_MUL 700
#define OTHER_ENG_MUL 500

#define BYTES_PER_PACKET 6
#define START_MARKER 0xFF
#define CONSOLE_REQUEST 0x61

#define RPM_2_DOTS 35000
#define RPM_3_DOTS 40000
#define RPM_4_DOTS 45000
#define RPM_5_DOTS 50000
#define RPM_6_DOTS 55000
#define RPM_7_DOTS 60000
#define RPM_8_DOTS 63000
#define RPM_9_DOTS 64000

#define EGT_1_DOT 100
#define EGT_2_DOTS 300
#define EGT_3_DOTS 825
#define EGT_4_DOTS 875

#define EGT_MUL 4.6
#define EGT_SUB 50
#define EDR_UPD_RATE_MS 5000
#define EDR_FIRST_STEP_DELAY_SEC 180
#define EDR_NEXT_STEP_DELAY_SEC 30

#define CONFIG_MODE_LED_UPD_MS 1000
#define TERM_INPUT_STR_LEN 30

#define DEF_GREEN_BR 6
#define DEF_ORANGE_BR 12
#define DEF_RED_BR 12
#define DEF_BAUDRATE 1

#define DATA1_START_CLEARANCE 0x8
#define DATA1_STARTING 0x10
#define DATA1_STARTED_UP 0x20
#define DATA1_IDLE 0x40
#define NOT_OP_SWITCH 0xF8

enum enum_SysState
{
  sysTest,
  sysConfiguring,
  sysNormal,
  sysFailure
};

typedef struct struct_EepromParams
{       
    uint8_t red;
    uint8_t green;
    uint8_t orange;
    uint8_t baudrate;
} EepromParamsType;


volatile uint8_t le;
volatile uint8_t re;
volatile uint8_t edr;
volatile uint8_t legt;
volatile uint8_t regt;
volatile uint8_t test_num;
volatile uint8_t sys_state;
volatile uint8_t is_le_mercury;
volatile uint8_t is_re_mercury;
volatile unsigned long start_time;
volatile unsigned long last_time_check;
String term_cmd;
volatile uint8_t char_cntr;
EepromParamsType eepromParams EEMEM;

#endif


