/*
 Pilot Display
 
 Copyright (c) 2014 Dmitry Pakhomenko.
 dmitryp@magictale.com
 http://magictale.com
 
 This code is in the public domain.
 */

#include <SPI.h>
#include <util/delay.h>
#include <Board.h>
#include <avr/eeprom.h>

#include "PilotDisplay.h"
#include "Messages.h"
#include "Utils.h"

//#define DEBUG
//#define GLOBAL_CURRENT_CTRL

void setup()
{
  
  eeprom_busy_wait();
  uint8_t br = eeprom_read_byte(&eepromParams.baudrate);
  if (br < 1 || br > 7) store_default_settings();

  eeprom_busy_wait();  
  br = eeprom_read_byte(&eepromParams.baudrate);  
  if (br == 1)
  {  
      Serial.begin(2400);
      Serial1.begin(2400);
  }else if (br == 2)
  {
      Serial.begin(4800);
      Serial1.begin(4800);
  }else if (br == 3)
  {
      Serial.begin(9600);
      Serial1.begin(9600);
  }else if (br == 4)
  {
      Serial.begin(19200);
      Serial1.begin(19200);
  }else if (br == 5)
  {
      Serial.begin(38400);
      Serial1.begin(38400);
  }else if (br == 6)
  {
      Serial.begin(57600);
      Serial1.begin(57600);
  }else
  {
      Serial.begin(115200);
      Serial1.begin(115200);
  }
  
  
  pinMode (slaveSelectPin, OUTPUT);
  SPI.begin();
  
  //configure all GPIO as current sinking outputs
  max6957_send_cmd(0xB, 0);
  max6957_send_cmd(0xC, 0);
  max6957_send_cmd(0xD, 0);  
  max6957_send_cmd(0xE, 0);  
  max6957_send_cmd(0xF, 0);  

#ifdef GLOBAL_CURRENT_CTRL
  max6957_send_cmd(MAX6957_CONFIG_REG, 1 << 6);
  max6957_send_cmd(MAX6957_GLOBAL_CURRENT_REG, 7); //7/16, ~10.5mA
  max6957_send_cmd(MAX6957_GLOBAL_CURRENT_REG, MAX_LED_BRIGHTNESS); //13/16, ~19.5mA
#else
  //Normal operation, individual current control
  max6957_send_cmd(MAX6957_CONFIG_REG, 1 << 6 | 0x1);
  
  eeprom_busy_wait();  
  uint8_t green = eeprom_read_byte(&eepromParams.green);  
  eeprom_busy_wait();  
  uint8_t orange = eeprom_read_byte(&eepromParams.orange);  
  eeprom_busy_wait();    
  uint8_t red = eeprom_read_byte(&eepromParams.red);    
  
  setup_LED_current(green, orange, red);
  
  //setup_LED_current(DEF_GREEN_BR, DEF_ORANGE_BR, DEF_RED_BR);
#endif

  sys_state = sysTest;
  le = 0;
  re = 0;
  edr = 0;
  legt = 0;
  regt = 0;
  
  term_cmd.reserve(TERM_INPUT_STR_LEN);
  char_cntr = 0;

  led_test();
  
  sys_state = sysNormal;
  
  start_time = millis();
  last_time_check = start_time;
}

void handle_cmd()
{
  if (term_cmd.startsWith("q\r") || term_cmd.startsWith("q\n"))
  {
      serial_println_p(NORMAL_MODE);
      sys_state = sysNormal;            
      edr = 0;
      render_display();
  }else if (term_cmd.startsWith("g "))
  {
      uint8_t arg = strtoint(term_cmd, 2);    
      if (arg >= 0 && arg <= 15)
      {
          serial_print_p(GREEN);
          serial_print_p(BRIGHTNESS);
          serial_print_p(SPACE_CHAR);
          Serial.println(arg);
          
          eeprom_busy_wait();
          uint8_t green = eeprom_read_byte(&eepromParams.green);
          if (green != arg)
          {
              eeprom_busy_wait();
              eeprom_write_byte(&eepromParams.green, arg);
          }
      }else serial_println_p(ARG_OUT_OF_RANGE);
  }else if (term_cmd.startsWith("o "))
  {
      uint8_t arg = strtoint(term_cmd, 2);    
      if (arg >= 0 && arg <= 15)
      {
          serial_print_p(ORANGE);
          serial_print_p(BRIGHTNESS);
          serial_print_p(SPACE_CHAR);
          Serial.println(arg);
          
          eeprom_busy_wait();
          uint8_t orange = eeprom_read_byte(&eepromParams.orange);
          if (orange != arg)
          {
              eeprom_busy_wait();
              eeprom_write_byte(&eepromParams.orange, arg);
          }
      }else serial_println_p(ARG_OUT_OF_RANGE);
  }else if (term_cmd.startsWith("r "))
  {
      uint8_t arg = strtoint(term_cmd, 2);    
      if (arg >= 0 && arg <= 15)
      {
          serial_print_p(RED);
          serial_print_p(BRIGHTNESS);
          serial_print_p(SPACE_CHAR);
          Serial.println(arg);
          
          eeprom_busy_wait();
          uint8_t red = eeprom_read_byte(&eepromParams.red);
          if (red != arg)
          {
              eeprom_busy_wait();
              eeprom_write_byte(&eepromParams.red, arg);
          }
      }else serial_println_p(ARG_OUT_OF_RANGE);
  }else if (term_cmd.startsWith("br "))
  {
      char arg = term_cmd.charAt(3);
      uint8_t idx = arg - 0x30;
      if (idx >= 1 && idx <= 7)
      {
          serial_print_p(BAUDRATE);  
          serial_print_p(SPACE_CHAR);            
          Serial.println(idx, DEC);
          
          eeprom_busy_wait();
          uint8_t br = eeprom_read_byte(&eepromParams.baudrate);
          if (br != idx)
          {
              eeprom_busy_wait();
              eeprom_write_byte(&eepromParams.baudrate, idx);
          }
      }else serial_println_p(ARG_OUT_OF_RANGE);
  }else if (term_cmd.startsWith("fr\r") || term_cmd.startsWith("fr\n"))
  {
      store_default_settings();
      serial_println_p(FACTORY_RESET);  
  }else if (term_cmd.startsWith("tst\r") || term_cmd.startsWith("tst\n"))
  {
      legt = 4;
      regt = 4;
      render_display();
      
      eeprom_busy_wait();  
      uint8_t green = eeprom_read_byte(&eepromParams.green);  
      eeprom_busy_wait();  
      uint8_t orange = eeprom_read_byte(&eepromParams.orange);  
      eeprom_busy_wait();    
      uint8_t red = eeprom_read_byte(&eepromParams.red);    
  
      setup_LED_current(green, orange, red);
  }else{
      serial_println_p(ERROR);
  }

  if (sys_state != sysNormal) serial_print_p(CMD_PROMT);  
}

void loop()
{  
  if (sys_state == sysNormal)
  {
  
    //Left Engine
    if (Serial.available())
    {
        if (Serial.peek() == CONSOLE_REQUEST)
        {
          if (Serial.available() >= BYTES_PER_PACKET)
          {
              Serial.read(); //dump first byte
              bool valid_req = true;
              for (uint8_t i = 0; i < BYTES_PER_PACKET - 1; i++)
              {
                  if (Serial.read() != CONSOLE_REQUEST)
                  {
                      valid_req = false;
                      break;
                  }
              }
              if (valid_req)
              {
                 usage();
                 sys_state = sysConfiguring;
              }
          }
        }else if (Serial.peek() == START_MARKER)
        {
            if (Serial.available() >= BYTES_PER_PACKET)
            {
              Serial.read(); //read the packet's start marker
              uint8_t data1 = Serial.read();  //read data1
            
              if (is_alternative_data_set(data1))
              {
#ifdef DEBUG
                serial_println_p(ALT);
#endif
                if (data1 == MERCURY_OLDER_ENGINE_ID ||
                    data1 == MERCURY_ENGINE_ID) is_le_mercury = true;
                else is_le_mercury = false;
              
                Serial.read(); //dump data2
                Serial.read(); //dump data3
                Serial.read(); //dump data4
                Serial.read(); //dump data5
              
              }else{
#ifdef DEBUG
                serial_println_p(NORM);
#endif
                uint32_t data2 = Serial.read(); //data2
                uint32_t rpm;
                if (is_le_mercury) rpm = data2 * MERCURY_MUL;
                else rpm =  data2 * OTHER_ENG_MUL;
              
#ifdef DEBUG
                Serial.print(data2, DEC);
                Serial.print(" ");
                Serial.println(rpm, DEC);
#endif
                le = 0;
                if (rpm >= RPM_9_DOTS) le = 9;
                else if (rpm >= RPM_8_DOTS) le = 8;
                else if (rpm >= RPM_7_DOTS) le = 7;
                else if (rpm >= RPM_6_DOTS) le = 6;
                else if (rpm >= RPM_5_DOTS) le = 5;
                else if (rpm >= RPM_4_DOTS) le = 4;
                else if (rpm >= RPM_3_DOTS) le = 3;
                else if (rpm >= RPM_2_DOTS) le = 2;
                else if ((data1 & NOT_OP_SWITCH) == DATA1_IDLE) le = 1;
                else if (((data1 & NOT_OP_SWITCH) == DATA1_START_CLEARANCE) ||
                         ((data1 & NOT_OP_SWITCH) == DATA1_STARTING) ||
                         ((data1 & NOT_OP_SWITCH) == DATA1_STARTED_UP)) le = E_IDLE;
                

#ifdef DEBUG
                Serial.print("LE=");
                Serial.println(le, DEC);
#endif

                if ((data1 & NOT_OP_SWITCH) == DATA1_START_CLEARANCE ||
                    (data1 & NOT_OP_SWITCH) == DATA1_STARTING ||
                    (data1 & NOT_OP_SWITCH) == DATA1_STARTED_UP ||
                    (data1 & NOT_OP_SWITCH) == DATA1_IDLE)
                    start_time = millis();                    

                uint16_t egt = 0;
                uint8_t data3 = Serial.read(); //data3
                if (data3 > 11) //Corner case, avoiding overflow if negative
                    egt = (double)data3 * EGT_MUL - EGT_SUB;
                
                legt = 0;

                if (egt >= EGT_4_DOTS) legt = 4;              
                else if (egt >= EGT_3_DOTS) legt = 3;
                else if (egt >= EGT_2_DOTS) legt = 2;              
                else if (egt >= EGT_1_DOT) legt = 1;

                Serial.read(); //dump data4
                Serial.read(); //dump data5
              
                render_display();
              }
            }  
        }else Serial.read();
    }
      
    //Right Engine
    if (Serial1.available())
    {
        if (Serial1.peek() == START_MARKER)
        {
            if (Serial1.available() >= BYTES_PER_PACKET)
            {
              Serial1.read(); //read the packet's start marker
              uint8_t data1 = Serial1.read();  //read data1
            
              if (is_alternative_data_set(data1))
              {
#ifdef DEBUG
                serial_println_p(ALT);
#endif
                if (data1 == MERCURY_OLDER_ENGINE_ID ||
                    data1 == MERCURY_ENGINE_ID) is_re_mercury = true;
                else is_re_mercury = false;
              
                Serial1.read(); //dump data2
                Serial1.read(); //dump data3
                Serial1.read(); //dump data4
                Serial1.read(); //dump data5
              
            }else{
#ifdef DEBUG
                serial_println_p(NORM);
#endif
                uint32_t data2 = Serial1.read(); //data2
                uint32_t rpm;
                if (is_re_mercury) rpm = data2 * MERCURY_MUL;
                else rpm =  data2 * OTHER_ENG_MUL;
              
#ifdef DEBUG
                Serial.print(data2, DEC);
                Serial.print(" ");
                Serial.println(rpm, DEC);
#endif
                re = 0;
                if (rpm >= RPM_9_DOTS) re = 9;
                else if (rpm >= RPM_8_DOTS) re = 8;
                else if (rpm >= RPM_7_DOTS) re = 7;
                else if (rpm >= RPM_6_DOTS) re = 6;
                else if (rpm >= RPM_5_DOTS) re = 5;
                else if (rpm >= RPM_4_DOTS) re = 4;
                else if (rpm >= RPM_3_DOTS) re = 3;
                else if (rpm >= RPM_2_DOTS) re = 2;
                else if ((data1 & NOT_OP_SWITCH) == DATA1_IDLE) re = 1;
                else if (((data1 & NOT_OP_SWITCH) == DATA1_START_CLEARANCE) ||
                         ((data1 & NOT_OP_SWITCH) == DATA1_STARTING) ||
                         ((data1 & NOT_OP_SWITCH) == DATA1_STARTED_UP)) re = E_IDLE;

#ifdef DEBUG
                Serial.print("RE=");
                Serial.println(re, DEC);
#endif
                if ((data1 & NOT_OP_SWITCH) == DATA1_START_CLEARANCE ||
                    (data1 & NOT_OP_SWITCH) == DATA1_STARTING ||
                    (data1 & NOT_OP_SWITCH) == DATA1_STARTED_UP ||
                    (data1 & NOT_OP_SWITCH) == DATA1_IDLE)
                    start_time = millis();                    
                    
                uint16_t egt = 0;
                uint8_t data3 = Serial1.read(); //data3
                if (data3 > 11) //Corner case, avoiding overflow if negative
                    egt = (double)data3 * EGT_MUL - EGT_SUB;
                
                regt = 0;

                if (egt >= EGT_4_DOTS) regt = 4;              
                else if (egt >= EGT_3_DOTS) regt = 3;
                else if (egt >= EGT_2_DOTS) regt = 2;              
                else if (egt >= EGT_1_DOT) regt = 1;

                Serial1.read(); //dump data4
                Serial1.read(); //dump data5
              
                render_display();
              }
            }  
        }else Serial1.read();
    }
  
    //Endurance
    if ((millis() - last_time_check) > EDR_UPD_RATE_MS)
    {
        unsigned long passed_time_sec = (millis() - start_time) / 1000; 
        if (passed_time_sec <= EDR_FIRST_STEP_DELAY_SEC) edr = 0;
        else edr = 1 + (passed_time_sec - EDR_FIRST_STEP_DELAY_SEC) / EDR_NEXT_STEP_DELAY_SEC;
        if (edr > 10) edr = 10;
        render_display();      
        last_time_check = millis(); 
    }
  
  }else if (sys_state = sysConfiguring)
  {
    while (Serial.available())
    {
        if (char_cntr < TERM_INPUT_STR_LEN)
        {
            char inChar = (char)Serial.read();
            term_cmd += inChar;
            if (inChar == '\n' || inChar == '\r')
            {
                handle_cmd();
                term_cmd = "";
            }
            Serial.print(inChar);
            char_cntr++;
        }else{
            term_cmd = "";
            char_cntr = 0;  
        }
    }

    //Configuration mode indication
    if ((millis() - last_time_check) > CONFIG_MODE_LED_UPD_MS)
    {
        if (edr != 0) edr = 0; 
        else edr = 1;
        
        render_display();
        last_time_check = millis(); 
    }
  }
}

