/*
 Pilot Display
 
 Copyright (c) 2014 Dmitry Pakhomenko.
 dmitryp@magictale.com
 http://magictale.com
 
 This code is in the public domain.
 */

#ifndef Utils_h
#define Utils_h

#include "PilotDisplay.h"

void serial_print_p(const prog_char str[])
{
  char c;
  if(!str) return;
  while((c = pgm_read_byte(str++)))
    Serial.write(c);
}

void serial_println_p(const prog_char str[])
{
  serial_print_p(str);
  Serial.println();
}

void max6957_send_cmd(uint8_t cmd, uint8_t data)
{
  digitalWrite(slaveSelectPin, LOW);
  SPI.transfer(cmd);
  SPI.transfer(data);
  SPI.transfer(cmd);
  SPI.transfer(data);
  digitalWrite(slaveSelectPin, HIGH); 
}

void max6957_send_cmd(uint8_t cmd1, uint8_t data1, uint8_t cmd2, uint8_t data2)
{
  digitalWrite(slaveSelectPin, LOW);
  SPI.transfer(cmd2);
  SPI.transfer(data2);
  SPI.transfer(cmd1);
  SPI.transfer(data1);
  digitalWrite(slaveSelectPin, HIGH); 
}

uint16_t num2bitval(uint8_t num)
{
  uint16_t res;
  uint8_t cntr = 0;
  while (num > 0)
  {
    res |= 1 << cntr;
    cntr++;
    num--;
  }
  return res;
}

uint8_t is_alternative_data_set(uint8_t data1)
{
  if (data1 == PEGASUS_OLDER_ENGINE_ID ||
      data1 == OLYMPUS_OLDER_ENGINE_ID ||
      data1 == MERCURY_OLDER_ENGINE_ID ||
      data1 == MERCURY_ENGINE_ID ||
      data1 == PEGASUS_ENGINE_ID ||
      data1 == OLYMPUS_ENGINE_ID ||
      data1 == TITAN_ENGINE_ID ||
      data1 == NIKE_ENGINE_ID ||
      data1 == LYNX_ENGINE_ID)  return true;
  else return false;
}

void setup_LED_current(uint8_t green, uint8_t orange, uint8_t red)
{
  max6957_send_cmd(0x12, (green & 0xF) | (green << 4 & 0xF0));
  max6957_send_cmd(0x13, (green & 0xF) | (green << 4 & 0xF0));
  max6957_send_cmd(0x14, (green & 0xF) | (green << 4 & 0xF0));
  max6957_send_cmd(0x15, (green & 0xF) | (green << 4 & 0xF0));
  max6957_send_cmd(0x16, (green & 0xF) | (green << 4 & 0xF0));
  max6957_send_cmd(0x17, (green & 0xF) | (green << 4 & 0xF0));
  max6957_send_cmd(0x18, (green & 0xF) | (green << 4 & 0xF0));
  max6957_send_cmd(0x19, (green & 0xF) | (green << 4 & 0xF0));
  max6957_send_cmd(0x1A, (orange & 0xF) | (red << 4 & 0xF0));
  max6957_send_cmd(0x1B, (green & 0xF) | (orange << 4 & 0xF0), 0x1B, (green & 0xF) | (green << 4 & 0xF0));
  max6957_send_cmd(0x1C, (orange & 0xF) | (red << 4 & 0xF0), 0x1C, (green & 0xF) | (green << 4 & 0xF0));  
  max6957_send_cmd(0x1D, (green & 0xF) | (orange << 4 & 0xF0), 0x1D, (green & 0xF) | (green << 4 & 0xF0));    
  max6957_send_cmd(0x1E, (orange & 0xF) | (red << 4 & 0xF0), 0x1E, (green & 0xF) | (green << 4 & 0xF0));    
  max6957_send_cmd(0x1F, (green & 0xF) | (green << 4 & 0xF0));
}  

void render_display()
{
  uint16_t res1;
  if (le == E_IDLE)
  {
      res1 = 1 << 8;
  }else
  {
      if (le > MAX_E) le = MAX_E;
      res1 = num2bitval(le);
  }
  
  uint16_t res2;
  if (re == E_IDLE)
  {
      res2 = 1 << 8;    
  }else
  {
      if (re > MAX_E) re = MAX_E;
      res2 = num2bitval(re);
  }
  
  if (edr > MAX_EDR) edr = MAX_EDR;
  uint16_t res3 = num2bitval(edr);  
  
  if (legt > MAX_EGT) legt = MAX_EGT;
  uint16_t res4 = num2bitval(legt);

  if (regt > MAX_EGT) regt = MAX_EGT;
  uint16_t res5 = num2bitval(regt);
 
  max6957_send_cmd(0x4D, (uint8_t)(res2 & 0xFF), 0x4D, (uint8_t)(res1 & 0xFF)); 
  
  max6957_send_cmd(0x55, (uint8_t)((res2 >> 8) & 0xFF) | (uint8_t)((res4 << 1) & 0x1E) | (uint8_t)((res5 << 5) & 0xE0), 
    0x55, (uint8_t)((res1 >> 8) & 0xFF) | (uint8_t)((res3 << 1) & 0xFE));
    
  max6957_send_cmd(0x5D, (uint8_t)((res5 >> 3) & 0x1), 0x5D, (uint8_t)((res3 >> 7) & 0x7));

}

void led_test()
{
  //LED test step 1
  max6957_send_cmd(MAX6957_GLOBAL_CURRENT_REG, MAX_LED_BRIGHTNESS);
  while (le <= MAX_E)
  {
    render_display();
    le++;
    re++;
    delay(DELAY_IN_TEST_MS);
  }
  while (le > 0)
  {
    render_display();
    le--;
    re--;
    delay(DELAY_IN_TEST_MS);
  }
  le = 0;
  re = 0;
  render_display();
  delay(DELAY_IN_TEST_MS);  
  
  //LED test step 2  
  while (edr <= MAX_EDR)
  {
    render_display();
    edr++;
    delay(DELAY_IN_TEST_MS);
  }
  while (edr > 0)
  {
    render_display();
    edr--;
    delay(DELAY_IN_TEST_MS);
  }
  edr = 0;
  render_display();
  delay(DELAY_IN_TEST_MS);
  
  //LED test step 3
  while (legt <= MAX_EGT)
  {
    render_display();
    legt++;
    regt++;
    delay(DELAY_IN_TEST_MS);
  }
  while (legt > 0)
  {
    render_display();
    legt--;
    regt--;
    delay(DELAY_IN_TEST_MS);
  }
  legt = 0;
  regt = 0;
  render_display();
  delay(DELAY_IN_TEST_MS);  
  
  // All LEDs are on
  /*max6957_send_cmd(0x4D, 0xFF);
  max6957_send_cmd(0x55, 0xFF);
  max6957_send_cmd(0x5D, 0xFF);
  
  uint8_t brightness = 0;
  while(++brightness <= MAX_LED_BRIGHTNESS)
  {
    max6957_send_cmd(MAX6957_GLOBAL_CURRENT_REG, brightness);
    delay(20);
  }
  
  delay(500);  
  
  while(--brightness > 0)
  {
    max6957_send_cmd(MAX6957_GLOBAL_CURRENT_REG, brightness);
    delay(20);
  }
  delay(500);*/
  
  //Brightness adj
  le = MAX_E;
  re = MAX_E;
  edr = MAX_EDR;
  legt = MAX_EGT;
  regt = MAX_EGT;
  
  render_display();
  delay(500);

  le = 0;
  re = 0;
  edr = 0;
  legt = 0;
  regt = 0;
  render_display();
  delay(500); 
}

void usage()
{
  //Switching to configuration mode
  serial_print_p(PRODUCT_NAME);
  serial_print_p(SPACE_CHAR);
  serial_print_p(FIRMWARE_REV);
  serial_print_p(SPACE_CHAR);
  serial_println_p(FIRMWARE_DATE);
  serial_println_p(COMPANY_URL);  
  serial_println_p(EQEQ27);
               
  serial_println_p(CONFIG_MODE);
  serial_println_p(USAGE1);
  serial_println_p(USAGE2);
  serial_println_p(USAGE3);
  serial_println_p(USAGE4);
  serial_println_p(USAGE5);
  serial_println_p(USAGE6);
  serial_println_p(USAGE7);                 
  
  serial_print_p(CURRENT_SETTINGS);    
  
  serial_print_p(BAUDRATE);  
  serial_print_p(SPACE_CHAR);            
  eeprom_busy_wait();  
  uint8_t br = eeprom_read_byte(&eepromParams.baudrate);
  Serial.print(br, DEC);          
  
  serial_print_p(GREEN);
  serial_print_p(BRIGHTNESS);  
  eeprom_busy_wait();  
  uint8_t green = eeprom_read_byte(&eepromParams.green);
  serial_print_p(SPACE_CHAR);  
  Serial.print(green, DEC);          
  
  serial_print_p(RED);
  serial_print_p(BRIGHTNESS);  
  eeprom_busy_wait();  
  uint8_t red = eeprom_read_byte(&eepromParams.red);
  serial_print_p(SPACE_CHAR);  
  Serial.print(red, DEC);          
  
  serial_print_p(ORANGE);
  serial_print_p(BRIGHTNESS);  
  eeprom_busy_wait();  
  uint8_t orange = eeprom_read_byte(&eepromParams.orange);
  serial_print_p(SPACE_CHAR);              
  Serial.println(orange, DEC);
 
  serial_print_p(CMD_PROMT);
}

uint8_t strtoint(String str, uint8_t startidx)
{
    uint8_t l = 0;
    while(str.charAt(startidx) >= '0' && str.charAt(startidx) <= '9')
    {
	l = l * 10 + (str.charAt(startidx) - '0');
        startidx++;
    }
    return l;
}

void store_default_settings()
{
  eeprom_busy_wait();
  uint8_t orange = eeprom_read_byte(&eepromParams.orange);
  if (orange != DEF_ORANGE_BR)
  {
    eeprom_busy_wait();
    eeprom_write_byte(&eepromParams.orange, DEF_ORANGE_BR);
  }
  
  eeprom_busy_wait();
  uint8_t red = eeprom_read_byte(&eepromParams.red);
  if (red != DEF_RED_BR)
  {
    eeprom_busy_wait();
    eeprom_write_byte(&eepromParams.red, DEF_RED_BR);
  }
  
  eeprom_busy_wait();
  uint8_t green = eeprom_read_byte(&eepromParams.green);
  if (green != DEF_GREEN_BR)
  {
    eeprom_busy_wait();
    eeprom_write_byte(&eepromParams.green, DEF_GREEN_BR);
  }
  
  eeprom_busy_wait();
  uint8_t br = eeprom_read_byte(&eepromParams.baudrate);
  if (br != DEF_BAUDRATE)
  {
    eeprom_busy_wait();
    eeprom_write_byte(&eepromParams.baudrate, DEF_BAUDRATE);
  }
}


#endif


